import {
  Http,
  IDictionary,
  IEventParam,
  Metadata,
} from '@zeedhi/core';
import {
  Button,
  DialogService,
  Form,
  LoadingService,
  Modal,
  ModalService,
  Select,
  Date,
  SelectMultiple,
  Tabs,
  TextInput,
} from '@zeedhi/common';
import { TekGrid, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import moment from 'moment';

interface EvolutionRow {
  DESCRESULEVOL: string;
  IDATIVO: string;
  IDGANHAPERDE: string;
  IDGERPROXATIV: string;
  IDRESULTCASO: string;
  IDRESULTEVOLUCAO: string;
  IDRESULTOPORTUNI: string;
  NRORG: string;
  NRSEQRESULEVOL: string;
}

/**
 * Represents a class for managing CRM leads.
 */
export class CrmLeads {
  /**
   * Represents a modal element used for adding new leads.
   *
   * This property holds a reference to a modal element that is used for adding new leads.
   * It is of type Modal, which represents a modal window or dialog.
   *
   * @type {Modal} A modal element for adding new leads.
   */
  private ModalElementoAdd!: Modal;

  /**
   * Represents the currently selected or focused row.
   * Can be used to store information about the selected row in a table or other user interface.
   * @type {any}
   */
  private currentRow: any = {};

  /**
   * Represents the currently selected or focused row.
   * Can be used to store information about the selected row in a table or other user interface.
   * @type {any}
   */
  private currentRowEvolutionTab: any = {};

  /**
   * Represents the currently selected or focused row.
   * Can be used to store information about the selected row in a table or other user interface.
   * @type {any}
   */
  private currentRowTaskScheduleTab: any = {};

  /**
   * Represents the currently selected or focused row.
   * Can be used to store information about the selected row in a table or other user interface.
   * @type {any}
   */
  private currentRowInterlocutorTab: any = {};

  /**
   * Determines the visibility of the main tekGrid for leads.
   * This property controls whether the leads grid is visible in the user interface.
   * @type {boolean}
   */
  public mainLeadScreenVisibility: boolean = true;

  /**
  * Determines the visibility of the evolution frame.
  * This property controls whether the card component is visible in the user interface.
  * @type {boolean}
  */
  public evolutionFrameVisibility: boolean = true;

  /**
   * Determines the visibility of the evolution grid.
   * This property controls whether the evolution grid is visible in the user interface.
   * @type {boolean}
   */
  public toggleTabVisibilityEvolution: boolean = true;

  /**
   * Determines the visibility of the evolution grid.
   * This property controls whether the evolution grid is visible in the user interface.
   * @type {boolean}
   */
  public toggleTabVisibilityTaskSchedule: boolean = true;

  /**
   * Determines the visibility of the tab for attachments.
   * @type {boolean}
   */
  public toggleTabVisibilityAttachment: boolean = true;

  /**
   * Determines the visibility of the tab for attachments.
   * @type {boolean}
   */
  public toggleInternalTabVisibilityAttachment: boolean = false;

  /**
   * Determines the visibility of the qualification button.
   * This property controls whether the qualification button is visible in the user interface.
   * @type {boolean}
   */
  public buttonQualifiedyVisibility: boolean = true;

  /**
   * Determines the visibility of the disqualification button.
   * This property controls whether the disqualification button is visible in the user interface.
   * @type {boolean}
   */
  public buttonDesqualifiedyVisibility: boolean = true;

  /**
   * Determines the visibility of the lead frame.
   * This property controls whether the lead frame is visible in the user interface.
   * @type {boolean}
   */
  public leadFrameVisibility: boolean = true;

  /**
   * Determines the visibility of the frame for lead screen.
   * This property controls whether the frame for lead screen is visible in the user interface.
   * @type {boolean}
   */
  public frameLeadScreenVisibility: boolean = true;

  /**
   * Determines the visibility of the internal evolution frame.
   * This property controls whether the internal evolution frame is visible in the user interface.
   * @type {boolean}
   */
  public toggleInternalEvolutionFrameVisibility: boolean = false;

  /**
   * Determines the visibility of the internal task schedule frame.
   * This property controls whether the internal evolution frame is visible in the user interface.
   * @type {boolean}
   */
  public toggleInternalTaksScheduleFrameVisibility: boolean = false;

  /**
   * Determines the visibility of the internal task schedule frame.
   * This property controls whether the internal evolution frame is visible in the user interface.
   * @type {boolean}
   */
  public toggleInternalInterlocutorFrameVisibility: boolean = false;

  public updateFieldVisibility: boolean = false;

  /**
  * Retrieves the name of the lead from the current row.
  * Returns the value of the 'NMCANDIDATO' property from the current row, which represents the name of the lead.
  * @returns {string} The name of the lead from the current row.
  */
  get textFrameLeadName(): string {
    return this.currentRow.NMCANDIDATO;
  }

  get textInternalEvolutionName(): string {
    return this.currentRowEvolutionTab.DESCRESULEVOL;
  }

  get textInternalTaskSchedule(): string {
    return this.currentRowTaskScheduleTab.DSFILACONTATO;
  }

  /**
  * Gets inverse lead grid visibility.
  * @returns {boolean} The visibility of the lead frame.
  */
  get changeLeadFrameVisibility(): boolean {
    return !this.mainLeadScreenVisibility;
  }

  /**
  * Returns the visibility state of the internal evolution frame for service scope visualization.
  * If the internal evolution frame visibility is toggled on, the display style of the 'visualization-container' component is set to 'flex'.
  * @returns The visibility state of the internal evolution frame for service scope visualization.
  */
  public get getVisualizationEvolutionTab(): boolean {
    if (this.frameLeadScreenVisibility) {
      this.setDisplayFlex('visualization-container');
    }
    return this.frameLeadScreenVisibility;
  }

  /**
  * Retrieves the code of the currently logged operator.
  *
  * This method asynchronously fetches the code of the operator who is currently logged in.
  * It returns a Promise that resolves to a string representing the code of the logged operator.
  *
  * @returns {Promise<string>} A Promise that resolves to the code of the logged operator.
  */
  private async getLoggedOperator(): Promise<string> {
    let loggedOper: string = '';
    try {
      const response = await Utilities.getLoggedOperator();
      loggedOper = response.data.CDOPERADOR;
    } catch (error) {
      Utilities.showErrorMessage(error);
    }
    return loggedOper;
  }

  private obtainTekTreeGrid(nameOfTekTreeGrid: string): TekTreeGrid {
    return Metadata.getInstance<TekTreeGrid>(nameOfTekTreeGrid);
  }

  private form(nameOfForm: string): Form {
    return Metadata.getInstance<Form>(nameOfForm);
  }

  /**
   * Selects the specified tab within the metadata.
   *
   * @param {string} tabName - The name of the tab to select.
   * @returns {void}
   */
  private selectTab(tabName: string): void {
    // Get the instance of the tabs metadata
    const tabs = Metadata.getInstance<Tabs>(tabName);

    // Set the specified tab as the active tab
    tabs.activeTab = 0;
  }

  /**
  * Closes the modal for adding new leads.
  */
  public closeModal(): void {
    this.ModalElementoAdd.destroy();
  }

  /**
   * Restores visibility of the main lead screen.
   *
   * @returns void
   */
  public backTekGridLeads(): void {
    this.mainLeadScreenVisibility = true;
  }

  /**
  * Restores visibility of the evolution grid and hides the principal frame leads.
  *
  * @returns void
  */
  public backTekGridEvolution(): void {
    this.hidePrincipalFrameLeads(true);
    this.changeVisibilityInternalEvolutionFrame(false);
  }

  public backTekGridTaksSchedule(): void {
    this.hidePrincipalFrameLeads(true);
    this.changeVisibilityTaskScheduleFrame(false);
    this.changeVisibilityInternalEvolutionFrame(false);
  }

  public backTekGridInterlocutor(): void {
    this.hidePrincipalFrameLeads(true);
    this.changeVisibilityTaskScheduleFrame(false);
    this.changeVisibilityInterlocutorFrame(false);
    this.changeVisibilityInternalEvolutionFrame(false);
  }

  /**
  * Opens the leads frame and performs necessary actions.
  *
  * @param row The row data associated with the lead.
  * @returns void
  */
  public openLeadsFrame({ row }: any): void {
    this.hideLeadGrid(false);
    this.selectTab('tabsInformationLeads');
    this.currentRow = row;
    this.loadTabLeadsData(row);
    this.modifyVisibilityBasedOnLeadStatus(row);
    this.setInterlocutorBasedOnNrParcNegocio('internal-frame-evolution-NMINTERLOCUTOR');
  }

  /**
   * Opens the internal evolution frame for the provided row.
   * @param {any} row - The row object for which the internal evolution frame should be opened.
   * @return {boolean} visibility - Indicates whether the internal evolution frame is visible or not.
   */
  public openEvolutionInternalFrame({ row }: any): void {
    this.changeVisibilityInternalEvolutionFrame(true);
    this.hidePrincipalFrameLeads(false);
    this.selectTab('tabEvolutionData');
    this.currentRowEvolutionTab = row;
    this.setEvolutionDataTab(row);
  }

  public openTaksScheduleInternalFrame({ row }: any): void {
    this.currentRowTaskScheduleTab = row;
    this.setTaskScheduleDataTab(row);
    this.hidePrincipalFrameLeads(false);
    this.changeVisibilityTaskScheduleFrame(true);
    this.changeVisibilityInternalEvolutionFrame(false);
  }

  public openIntelocutorInternalFrame({ row }: any): void {
    this.currentRowInterlocutorTab = row;
    this.setInterlocutorDataTab(row);
    this.hidePrincipalFrameLeads(false);
    this.changeVisibilityInterlocutorFrame(true);
    this.changeVisibilityTaskScheduleFrame(false);
    this.changeVisibilityInternalEvolutionFrame(false);
  }

  /**
   * Changes the visibility of the internal evolution frame.
   * @param visibility - A boolean value indicating the visibility of the internal evolution frame.
   */
  private changeVisibilityInternalEvolutionFrame(visibility: boolean): void {
    this.toggleInternalEvolutionFrameVisibility = visibility;
  }

  /**
   * Hides or shows the lead grid based on the specified visibility.
   * @param visibility - A boolean value indicating whether to hide or show the lead grid.
   */
  private hideLeadGrid(visibility: boolean): void {
    this.mainLeadScreenVisibility = visibility;
  }

  /**
   * Hides or shows the principal frame for leads.
   * @param visibility - A boolean value indicating whether to hide or show the frame.
   */
  private hidePrincipalFrameLeads(visibility: boolean): void {
    this.frameLeadScreenVisibility = visibility;
  }

  private changeVisibilityTaskScheduleFrame(visibility: boolean): void {
    this.toggleInternalTaksScheduleFrameVisibility = visibility;
  }

  private changeVisibilityInterlocutorFrame(visibility: boolean): void {
    this.toggleInternalInterlocutorFrameVisibility = visibility;
  }

  private updateFieldVisibilityBasedOnTask(visibility: boolean): void {
    this.updateFieldVisibility = visibility;
  }

  /**
   * Loads the data into the tab leads based on the provided row object.
   * @param row - The row object containing the data to be loaded.
   */
  private loadTabLeadsData(row: any): void {
    Metadata.getInstance<TextInput>('formCandidateData-NRCANDIDATO').value = row.NRCANDIDATO;
    Metadata.getInstance<TextInput>('formCandidateData-INCLUSAO').value = row.INCLUSAO;
    Metadata.getInstance<Select>('formCandidateData-IDSTATUSCAN').value = row.IDSTATUSCAN;
    Metadata.getInstance<Select>('formCandidateData-IDATIVO').value = row.IDATIVO;
    Metadata.getInstance<TextInput>('formCandidateData-NMCANDIDATO').value = row.NMCANDIDATO;
    Metadata.getInstance<TextInput>('formCandidateData-NMFANTCAN').value = row.NMFANTCAN;
    Metadata.getInstance<TextInput>('formCandidateData-NRINSJURCAN').value = row.NRINSJURCAN;
    Metadata.getInstance<Select>('formCandidateData-IDTPIJURCAN').value = row.IDTPIJURCAN;
    Metadata.getInstance<TextInput>('formCandidateData-DSEMAILCAN').value = row.DSEMAILCAN;
    Metadata.getInstance<TextInput>('formCandidateData-DSTELEFONECAN').value = row.DSTELEFONECAN;
    Metadata.getInstance<Select>('formSegmentData-PERFATUSEGMER').value = row.PERFATUSEGMER;
    Metadata.getInstance<Select>('formSegmentData-CDSEGMER').value = row.CDSEGMER;
    Metadata.getInstance<TextInput>('formSegmentData-NRFUNCCAN').value = row.NRFUNCCAN;
    Metadata.getInstance<TextInput>('formSegmentData-PORTCANDIDATO').value = row.PORTCANDIDATO;
    Metadata.getInstance<TextInput>('formSegmentData-QNTDUSUARIOS').value = row.QNTDUSUARIOS;
    Metadata.getInstance<TextInput>('formSegmentData-NRSISATU').value = row.NRSISATU;
    Metadata.getInstance<TextInput>('formSegmentData-QNTPONTOSACESSO').value = row.QNTPONTOSACESSO;
    Metadata.getInstance<TextInput>('formSegmentData-FISCAL').value = row.FISCAL;
    Metadata.getInstance<TextInput>('formSegmentData-CDMODATEND').value = row.CDMODATEND;
    Metadata.getInstance<TextInput>('formSegmentData-QNTDUNIDADES').value = row.QNTDUNIDADES;
    Metadata.getInstance<TextInput>('formSegmentData-NRREFEICOESDIA').value = row.NRREFEICOESDIA;
    Metadata.getInstance<TextInput>('formAdressData-NRCEPENDERECO').value = row.NRCEPENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DSENDERECO').value = row.DSENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-NRIMOVELENDERECO').value = row.NRIMOVELENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DSCOMPLEENDERECO').value = row.DSCOMPLEENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DESCTITULOCAN').value = row.DESCTITULOCAN;
    Metadata.getInstance<TextInput>('formAdressData-CDPAIS').value = row.CDPAIS;
    Metadata.getInstance<TextInput>('formAdressData-SGESTADO').value = row.SGESTADO;
    Metadata.getInstance<TextInput>('formAdressData-CDMUNICIPIO').value = row.CDMUNICIPIO;
    Metadata.getInstance<TextInput>('formAdressData-CDBAIRRO').value = row.CDBAIRRO;
    Metadata.getInstance<Select>('formDadosAdicionais-CDOPERADOR').value = row.CDOPERADOR;
    Metadata.getInstance<Select>('formDadosAdicionais-CDMODODIVULG').value = row.CDMODODIVULG;
    Metadata.getInstance<Select>('formDadosAdicionais-NMMODORECEBINFO').value = row.NMMODORECEBINFO;
    Metadata.getInstance<TextInput>('formDadosAdicionais-DSOBSERVACAOLEAD').value = row.DSOBSERVACAOLEAD;
  }

  /**
   * Sets evolution data tab values based on the provided row object.
   * @param row - The row object containing data for the evolution tab.
   */
  private setEvolutionDataTab(row: any): void {
    Metadata.getInstance<TextInput>('internal-frame-evolution-DTHREVOLCONT').value = row.DTHREVOLCONT;
    Metadata.getInstance<TextInput>('internal-frame-evolution-IDSENEVOCONT').value = row.IDSENEVOCONT;
    Metadata.getInstance<TextInput>('internal-frame-evolution-CDOPEREVOL').value = row.CDOPEREVOL;
    Metadata.getInstance<Select>('internal-frame-evolution-CDTIPOCONT').value = row.CDTIPOCONT;
    Metadata.getInstance<Select>('internal-frame-evolution-CDMOTIEVOCON').value = row.CDMOTIEVOCON;
    Metadata.getInstance<TextInput>('internal-frame-evolution-NRSEQRESULEVOL').value = row.NRSEQRESULEVOL;
    Metadata.getInstance<TextInput>('internal-frame-evolution-NMINTERLOCUTOR').value = row.NRSEQINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-frame-evolution-TXEVOLCONT').value = row.TXEVOLCONT;
    Metadata.getInstance<TextInput>('formTaskscheduleData-DTPLANFILACONTA').value = row.DTPLANFILACONTA;
    Metadata.getInstance<TextInput>('internal-frame-evolution-CDOPERREAL').value = row.CDOPERREAL;
    Metadata.getInstance<TextInput>('internal-frame-evolution-NRSEQCLASSIFICATAREFA').value = row.NRSEQCLASSIFICATAREFA;
    Metadata.getInstance<TextInput>('internal-frame-evolution-DSFILACONTATO').value = row.DSFILACONTATO;
    Metadata.getInstance<TextInput>('internal-frame-evolution-NRPRIORFILACONT').value = row.NRPRIORFILACONT;
  }

  /**
   * Sets task schedule data tab values based on the provided row object.
   * @param row - The row object containing data for the task schedule tab.
   */
  private setTaskScheduleDataTab(row: any): void {
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DSFILACONTATO').value = row.DSFILACONTATO;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-IDATIVO').value = row.IDATIVO;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-NRSEQCLASSIFICATAREFA').value = row.NRSEQCLASSIFICATAREFA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DTHRPLANFILACONTA').value = row.DTHRPLANFILACONTA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DTVENCFILACONTA').value = row.DTVENCFILACONTA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-CDOPERREAL').value = row.CDOPERREAL;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DTREALFILACONTA').value = row.DTREALFILACONTA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-NRPRIORFILACONT').value = row.NRPRIORFILACONT;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-NMINTERLOCUTOR').value = row.NMINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DSTELEFONETAREFA').value = row.DSTELEFONETAREFA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DSEMAILTAREFA').value = row.DSEMAILTAREFA;
    Metadata.getInstance<TextInput>('internal_frame_task_schedule-DSDETEFILACONTA').value = row.DSDETEFILACONTA;
  }

  private manageNextTaskFieldsRequirement() {
    if (this.updateFieldVisibility) {
      Metadata.getInstance<TextInput>('modal-register-evolution-DTPLANFILACONTA').addValidation('required', {});
      Metadata.getInstance<Select>('modal-register-evolution-CDOPERREAL').addValidation('required', {});
      Metadata.getInstance<Select>('modal-register-evolution-NRSEQCLASSIFICATAREFA').addValidation('required', {});
      Metadata.getInstance<TextInput>('modal-register-evolution-DSFILACONTATO').addValidation('required', {});
      Metadata.getInstance<Select>('modal-register-evolution-NRPRIORFILACONT').addValidation('required', {});
    } else {
      Metadata.getInstance<TextInput>('modal-register-evolution-DTPLANFILACONTA').removeValidation('required');
      Metadata.getInstance<Select>('modal-register-evolution-CDOPERREAL').removeValidation('required');
      Metadata.getInstance<Select>('modal-register-evolution-NRSEQCLASSIFICATAREFA').removeValidation('required');
      Metadata.getInstance<TextInput>('modal-register-evolution-DSFILACONTATO').removeValidation('required');
      Metadata.getInstance<Select>('modal-register-evolution-NRPRIORFILACONT').removeValidation('required');
    }
  }

  /**
   * Sets data for the interlocutor tab.
   *
   * @param row An object containing the data for the interlocutor.
   */
  public setInterlocutorDataTab(row: any): void {
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMINTERLOCUTOR').value = row.NMINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NRCPFINTERLOC').value = row.NRCPFINTERLOC;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMSETOR').value = row.CDSETOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMCARGOINTERLOC').value = row.CDCARGOINTERLOC;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-EMAILINTERLOC').value = row.EMAILINTERLOC;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NRTELEFONE1').value = row.NRTELEFONE1;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NRRAMALTEL1').value = row.NRRAMALTEL1;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NRCEPINTERLOCUTOR').value = row.NRCEPINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-DSENDEINTERLOCUTOR').value = row.DSENDEINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-DSCOMPINTERLOCUTOR').value = row.DSCOMPINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NRENDEINTERLOCUTOR').value = row.NRENDEINTERLOCUTOR;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMPAIS').value = row.CDPAIS;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMESTADO').value = row.SGESTADO;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMMUNICIPIO').value = row.CDMUNICIPIO;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-NMBAIRRO').value = row.CDBAIRRO;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-DTNASCIMENTO').value = row.DTNASCIMENTO;
    Metadata.getInstance<TextInput>('internal-tab-interlocutor-IDRECEBEPRESENTE').value = row.IDRECEBEPRESENTE;
  }

  /**
   * Checks the active tab and performs specific actions based on the active tab.
   */
  public checkActiveTab(): void {
    // Get instance of the main TekGrid
    const mainTekGrid = Metadata.getInstance<TekGrid>('tek_grid_crm_leads');

    // Get instance of the Tabs component containing tab information
    const tabs = Metadata.getInstance<Tabs>('tabsInformationLeads');

    // Get the active tab object
    const activeTab = tabs.tabs[tabs.activeTab];

    // Switch statement to handle different active tabs
    switch (activeTab.name) {
      // If the active tab is the evolution tab
      case 'tab_evolution':
        // Load evolution tab items based on the current row of the main TekGrid
        this.loadEvolutionTabItems({ row: mainTekGrid.datasource.currentRow });
        break;
      // If the active tab is the task schedule tab
      case 'tab_task_schedule':
        // Load task schedule tab groups
        this.loadTaskScheduleTabGroups({ row: mainTekGrid.datasource.currentRow });
        break;
      // If the active tab is the interlocutor tab
      case 'tab_interlocutor':
        // Load interlocutor tab tasks
        this.loadInterlocutorTabTasks({ row: mainTekGrid.datasource.currentRow });
        break;
      // Default case (if the active tab is not recognized)
      default:
        // Do nothing
        break;
    }
  }

  /**
   * Validates and processes actions based on the active tab in the Evolution tab.
   * If the active tab is the 'tab_attachment', it loads items from the Attachment tab.
   * @returns {void}
   */
  public validateIntegratedTabsInEvolutionTab(): void {
    // Get the instance of the main TekGrid
    const mainTekGrid = Metadata.getInstance<TekGrid>('tek_grid_crm_leads');

    // Get the instance of the Evolution tab
    const tabs = Metadata.getInstance<Tabs>('tabEvolutionData');
    const activeTab = tabs.tabs[tabs.activeTab];

    // Process actions based on the active tab
    switch (activeTab.name) {
      case 'tab_attachment':
        // If the active tab is 'tab_attachment', load items from the Attachment tab
        this.loadAttachmentTabItems({ row: mainTekGrid.datasource.currentRow });
        break;
      default:
        // Other cases can be handled here if necessary
        break;
    }
  }

  /**
   * Loads items from the Evolution tab after the screen is built.
   * @param {Object} options - The options for loading the Evolution tab items.
   * @param {Object} options.row - The selected row data.
   * @returns {void}
   */
  private loadEvolutionTabItems({ row }: { row: any }): void {
    // Load items from the Evolution tab after the screen is built
    setTimeout(() => {
      // Get the instance of the Evolution grid
      const evolutionGrid = Metadata.getInstance<TekGrid>('tekgrid-tab-evolution');

      // Set the filter based on the selected row data
      evolutionGrid.datasource.setFilter({
        NRCONTATO: row.NRCONTATO,
        NRSEQCONTATO: row.NRSEQCONTATO,
      });

      // Load data from the Evolution grid's data source
      evolutionGrid.datasource.get();
    }, 700); // Wait time before loading the data (in milliseconds)
  }

  /**
   * Loads items from the Attachment tab after the screen is built.
   * This function sets a filter on the Attachment grid based on the selected row data
   * and then loads data from the Attachment grid's data source after a delay.
   * @param {Object} options - The options for loading the Attachment tab items.
   * @param {Object} options.row - The selected row data.
   * @returns {void}
   */
  private loadAttachmentTabItems({ row }: { row: any }): void {
    // Load items from the Attachment tab after the screen is built
    setTimeout(() => {
      // Get the instance of the Attachment grid
      const attachmentGrid = Metadata.getInstance<TekGrid>('tekgrid-tab-attachment-evolution');

      // Set the filter based on the selected row data
      attachmentGrid.datasource.setFilter({
        NRCONTATO: row.NRCONTATO,
        NRSEQCONTATO: row.NRSEQCONTATO,
      });

      // Load data from the Attachment grid's data source
      attachmentGrid.datasource.get();
    }, 700); // Wait time before loading the data (in milliseconds)
  }

  private loadTaskScheduleTabGroups({ row }: { row: any }): void {
    // Load items from the Attachment tab after the screen is built
    setTimeout(() => {
      // Get the instance of the Attachment grid
      const taskScheduleGrid = Metadata.getInstance<TekGrid>('tekgrid-task-schedule-tab');

      // Set the filter based on the selected row data
      taskScheduleGrid.datasource.setFilter({
        NRCONTATO: row.NRCONTATO,
        NRSEQCONTATO: row.NRSEQCONTATO,
      });

      // Load data from the Attachment grid's data source
      taskScheduleGrid.datasource.get();
    }, 700); // Wait time before loading the data (in milliseconds)
  }

  private loadInterlocutorTabTasks({ row }: { row: any }): void {
    // Load items from the Attachment tab after the screen is built
    setTimeout(() => {
      // Get the instance of the Attachment grid
      const interlocutorGrid = Metadata.getInstance<TekGrid>('tekgrid-interlocutor-tab');

      // Set the filter based on the selected row data
      interlocutorGrid.datasource.setFilter({
        NRPARCNEGOCIO: row.NRPARCNEGOCIO,
      });

      // Load data from the Attachment grid's data source
      interlocutorGrid.datasource.get();
    }, 700); // Wait time before loading the data (in milliseconds)
  }

  /**
   * Retrieves the instance of the TekGrid associated with CRM leads.
   * @returns The instance of the TekGrid associated with CRM leads.
   */
  private getGridInstance() {
    return Metadata.getInstance<TekGrid>('tek_grid_crm_leads');
  }

  /**
   * Retrieves the filter inputs for a specified field from the grid instance.
   *
   * This method retrieves the filter inputs associated with the specified field from the grid instance.
   * It returns the filter inputs for the specified field.
   *
   * @param {string} field - The name of the field for which to retrieve filter inputs.
   * @returns {any} The filter inputs associated with the specified field.
   */
  private getFilterInputs(field: string): any {
    return this.getGridInstance().getFilterInputs(field);
  }

  /**
   * Opens the filter modal associated with the specified component.
   *
   * This method triggers a click event on the filter button associated with the specified component,
   * opening the filter modal for that component.
   *
   * @param {any} component - The component for which to open the filter modal.
   */
  public openFilter({ component }: any): void {
    const filterButton = Metadata.getInstance<Button>(
      `${component.name}_filterButton`,
    );
    filterButton.click({} as Event, {} as HTMLElement);
  }

  /**
   * Adds a new lead by displaying a modal form.
   *
   * This method creates a modal form for adding a new lead and displays it to the user.
   *
   * @returns {Promise<void>} A promise that resolves once the modal is displayed.
   */
  public async addNewLead(): Promise<void> {
    try {
      // Create a modal form element for adding a new lead.
      this.ModalElementoAdd = await ModalService.createFromJson('/crm_leads/Modal/crm_leads_cadastro', true);

      // Display the modal form to the user.
      this.ModalElementoAdd.show();
    } catch (error) {
      // Log any errors encountered while adding a new lead.
      console.error('Error adding a new lead:', error);
    }
  }

  /**
   * Opens a modal form to add a new evolution entry for a lead.
   * This method creates a modal form element for adding a new evolution and displays it to the user.
   * @returns A Promise that resolves once the modal form is created and displayed.
   */
  public async addNewEvolution(): Promise<void> {
    try {
      // Create a modal form element for adding a new lead.
      this.ModalElementoAdd = await ModalService.createFromJson('/crm_leads/Modal/register_evolution', true);

      // Display the modal form to the user.
      this.ModalElementoAdd.show();
    } catch (error) {
      // Log any errors encountered while adding a new lead.
      console.error('Error adding a new lead:', error);
    }
  }

  /**
   * Opens a modal form to add a new task entry for a lead.
   * This method creates a modal form element for adding a new task and displays it to the user.
   * @returns A Promise that resolves once the modal form is created and displayed.
   */
  public async addNewTaskForLead(): Promise<void> {
    try {
      // Create a modal form element for adding a new lead.
      this.ModalElementoAdd = await ModalService.createFromJson('/crm_leads/Modal/register_new_schedule_task', true);

      // Display the modal form to the user.
      this.ModalElementoAdd.show();
    } catch (error) {
      // Log any errors encountered while adding a new lead.
      console.error('Error adding a new lead:', error);
    }
  }

  /**
  * Sets the NRPARCRESP filter based on the logged operator.
  * @param componentName The name of the component to obtain the instance from.
  * @returns A Promise that resolves once the filter is set.
  */
  private async setNRPARCRESP(componentName: string): Promise<void> {
    try {
      // Display the loading indicator
      LoadingService.show();

      // Obtain the instance of the logged operator's selection component
      const loggedOperInstance = Metadata.getInstance<Select>(componentName);

      // Obtain the code of the logged operator
      const cdOperador = await this.getLoggedOperator();

      // Set the value of the NRPARCRESP filter as the code of the logged operator
      loggedOperInstance.setValue(cdOperador);

      // Hide the loading indicator
      LoadingService.hide();
    } catch (error) {
      // Log the error to the console in case of failure
      console.error('Error setting filter:', error);
    }
  }

  /**
  * Sets the NRPARCRESP filter using the provided component.
  * @param component An object containing information about the component.
  */
  public getComponenteNameForFilterNRPARCRESP({ component }: any): void {
    this.setNRPARCRESP(component.name);
  }

  public getInterlocutorBasedOnNrParcNegocio({ component }: any): void {
    this.setInterlocutorBasedOnNrParcNegocio(component.name);
  }

  /**
  * Sets the NRPARCRESP filter using the provided component.
  */
  public setInterlocutorBasedOnNrParcNegocio(fieldName: string): void {
    const row = this.currentRow;

    // Get the instance of the Evolution grid
    const interlocutorField = Metadata.getInstance<Select>(fieldName);

    // Set the filter based on the selected row data
    interlocutorField.datasource.setFilter({
      NRPARCNEGOCIO: row.NRPARCNEGOCIO,
    });

    // Load data from the Evolution grid's data source
    interlocutorField.datasource.get();
  }

  /**
   * Controls the visibility based on the next task selected in the evolution description dropdown.
   */
  public visibilityControlBasedOnTheNextTask({ component }: any): void {
    this.changeVisibilityForTaskLabels(component.datasource.currentRowValue);
  }

  private changeVisibilityForTaskLabels(currentRow: EvolutionRow) {
    if (currentRow !== null && currentRow !== undefined) {
      if (currentRow.IDGERPROXATIV === 'S') {
        this.updateFieldVisibilityBasedOnTask(true);
      } else {
        this.updateFieldVisibilityBasedOnTask(false);
      }
    } else {
      this.updateFieldVisibilityBasedOnTask(false);
    }
    this.manageNextTaskFieldsRequirement();
  }

  /**
  * Performs a search to check if there is already a lead with the provided company name (NMCANDIDATO).
  * If a lead with the same company name exists, it displays an error message and clears the value of the input component.
  * @param component The text input component containing the company name to be checked.
  */
  public searchParcForName({ component }: any): void {
    // Check if the component value is not null
    if (component.value != null) {
      // Obtain the value of the company name from the text input component
      const NMCANDIDATO = Metadata.getInstance<TextInput>('modal-cadastro-leads-NMCANDIDATO').value;

      // Display the loading indicator
      LoadingService.show();

      // Perform an asynchronous request to validate the lead's company name
      Http.post('/leadValidate', {
        NMCANDIDATO: NMCANDIDATO.toString().replace(/ /g, '%').toUpperCase(),
      }).then((response: any) => {
        // Check if there is data in the response of the request
        if ((response.data.data.length !== 0)) {
          // Display an error message indicating that a lead with this company name already exists
          Utilities.showDialogMessage('error', 'Operation Blocked!', 'A lead with this Company Name already exists!');

          // Clear the value of the component
          component.value = null;
        }
      }).catch((error: any) => {
        // Display an error message in case of an error
        Utilities.showErrorMessage(error);
      }).finally(() => {
        // Hide the loading indicator after the request is completed
        LoadingService.hide();
      });
    }
  }

  /**
   * Determines the mask format based on the value selected in the 'NRINSJURCAN' field of the modal.
   *
   * This method retrieves the formatted value of the 'NRINSJURCAN' field from the modal and
   * determines the appropriate mask format based on the selected value.
   *
   * @returns {string} - The mask format determined based on the selected value in the 'NRINSJURCAN' field.
   */
  public getMaskModal(value: string): string {
    switch (value.length) {
      case 14:
        return '##.###.###/####-##';
      case 11:
        return '###.###.###-##';
      case 6:
        return '######'; // Alterei para 6 porque o CPF tem 11 dígitos, não 6.
      case 20:
        return 'NNNNNNNNNNNNNNNNNNNN'; // Corrigi para 20 dígitos, assumindo CNPJ como exemplo.
      default:
        return '';
    }
  }

  /**
   * Validates and formats the input value for CNPJ or CPF.
   *
   * This method validates the input value for CNPJ or CPF format and updates the corresponding fields accordingly.
   *
   * @param {IEventParam<TextInput>} component - The TextInput component triggering the event.
   */
  public checkFormatCNPJ({ component }: IEventParam<TextInput>): void {
    if (component.value != null && component.value.length > 0) {
      const formattedValue = component.value.replace(/[^0-9]/g, '');
      if (formattedValue.length === 14) {
        if (!Utilities.validateCNPJ(formattedValue)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'CNPJ inválido!');
          component.value = null;
        } else {
          Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value = 'J';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').value = formattedValue;
          this.validateRegistration(Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN'),
            Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN'));
        }
      } else if (formattedValue.length === 11) {
        // Validar CPF
        if (!Utilities.validateCPF(formattedValue)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'CPF inválido!');
          component.value = null;
        } else {
          Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value = 'F';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').mask = '###.###.###-##';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').value = formattedValue;
          this.validateRegistration(Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN'),
            Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN'));
        }
      }
      // else {
      //   // Validar padrões internacionais
      //   if (!Utilities.validateInternationalPattern(component.value)) {
      //     Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Formato inválido!');
      //     component.value = null;
      //   }
      // }
    } else {
      Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').mask = '';
    }
  }

  /**
   * Handles the click event of a button to expand or collapse a form.
   * @param {Object} options - The options object.
   * @param {any} options.component - The component object.
   * @returns {void}
   */
  public clickButtonToExpand({ component }: any): void {
    const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
    form.isVisible = !form.isVisible;
    component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
  }

  /**
   * Validates the component.
   *
   * @param {any} options - The options object.
   * @param {any} options.component - The component to validate.
   * @returns {void}
   */
  public validate({ component }: any): void {
    const nameForm = component.name.split('-')[0];
    this.validateCEP(`${nameForm}-NRCEPENDERECO`);
  }

  /**
   * Validates the registration of a partner based on the type of inscription and the inscription field value.
   *
   * This method validates the registration of a partner based on the
   * type of inscription (e.g., CNPJ or CPF)
   * and the corresponding inscription field value.
   * It sends a request to the server to check if there is already
   * a partner associated with the provided inscription. If a partner with the same inscription exists, it displays
   * an error message.
   *
   * @param {any} inscriptionField - The field containing the inscription value (e.g., NRINSJURCAN).
   * @param {any} typeIncription - The field containing the type of inscription (e.g., IDTPIJURCAN).
   */
  public validateRegistration(inscriptionField: any, typeIncription: any): void {
    if (inscriptionField.value != null && typeIncription.value != null) {
      const IDTPIJURCAN = typeIncription.value;
      const NRINSJURCAN = inscriptionField.value;
      LoadingService.show();
      Http.post('/leadValidate', {
        IDTPIJURCAN,
        NRINSJURCAN,
      }).then((response: any) => {
        if ((response.data.data.length !== 0)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um Parceiro atrelado a este CNPJ');
          inscriptionField.value = null;
        }
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  /**
   * Handles the event triggered when a filter grid is created.
   *
   * This method is called when a filter grid component is created,
   * allowing dynamic filtering of data.
   * It sets up dynamic filters based on the current user's operator
   * ID and the active status of the records.
   *
   * @param {any} component The created filter grid component object.
   * @returns {Promise<void>}
   */
  public async onCreatedFilterGrid({ component }: any): Promise<void> {
    try {
      (component.datasource as TekRestDatasource).dynamicFilter = {
        IDOPERATIVO: [{
          relation: 'AND',
          operation: 'EQUALS',
          value: 'A',
        }],
        IDATIVO: [{
          relation: 'AND',
          operation: 'EQUALS',
          value: 'S',
        }],
        CDOPERADOR: [{
          relation: 'AND',
          operation: 'IN',
          value: [localStorage.getItem('CDOPERADOR')],
        }],
      };
    } catch (error) {
      console.error('Erro ao definir dynamicFilter:', error);
    }
  }

  /**
   * Handles changes in the status of an operator.
   *
   * This method is triggered when there's a change in the status of an operator component.
   * It filters the options of the responsible operator field based on the selected status
   * of the operator.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */
  public changeStatusOper({ component }: any): void {
    const statusOper = this.getFilterInputs('IDOPERATIVO');
    const respOperator = this.getFilterInputs('CDOPERADOR');

    const respOperatorInstance = Metadata.getInstance<SelectMultiple>(respOperator[0].name);

    if (component.value) {
      respOperatorInstance.datasource.setFilter({
        IDOPERATIVO: statusOper[0].value,
      });
    } else {
      respOperatorInstance.datasource.setFilter({
        IDOPERATIVO: 'A',
      });
    }
  }

  /**
   * Handles changes in the state field based on the selected country.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the country field, it enables the state
   * field and filters its options based on the selected country.
   * If the changed component is the country field within a modal, it
   * behaves similarly to the previous case, but with modal-specific field instances.
   * If the changed component is neither the country field nor within a modal,
   * it disables the state field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */

  public changeFieldState({ component }: any): void {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');

    const stateFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldState[0].name);

    if (component.value != null && component.name === statusFieldCountry[0].name) {
      stateFieldInstance.disabled = false;
      stateFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-CDPAIS') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      modalFieldState.disabled = false;
      modalFieldState.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
      });
    } else {
      stateFieldInstance.disabled = true;
      stateFieldInstance.value = null;
    }
  }

  /**
   * Handles changes in the municipality field based on the selected state.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the state field, it enables the municipality field and filters
   * its options based on the selected country and state.
   * If the changed component is the state field within a modal, it behaves similarly to the
   * previous case, but with modal-specific field instances.
   * If the changed component is neither the state field nor within a modal,
   * it disables the municipality field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */
  public changeInTheMunicipalityField({ component }: any): void {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');
    const statusFieldMunicipality = this.getFilterInputs('CDMUNICIPIO');

    const municipalityFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldMunicipality[0].name);

    if (component.value != null && component.name === statusFieldState[0].name) {
      municipalityFieldInstance.disabled = false;
      municipalityFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
        SGESTADO: statusFieldState[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-SGESTADO') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      const modalFieldMunicipality = Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO');
      modalFieldMunicipality.disabled = false;
      modalFieldMunicipality.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
        SGESTADO: modalFieldState.value,
      });
    } else {
      municipalityFieldInstance.disabled = true;
      municipalityFieldInstance.value = null;
    }
  }

  /**
   * Handles changes in the neighborhood field based on the selected municipality.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the municipality field, it enables the neighborhood
   * field and filters its options based on the selected country, state, and municipality.
   * If the changed component is the municipality field within a modal, it behaves
   * similarly to the previous case, but with modal-specific field instances.
   * If the changed component is neither the municipality field nor within a modal,
   * it disables the neighborhood field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */
  public changeInTheNeighborhoodField({ component }: any): void {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');
    const statusFieldNeighborhood = this.getFilterInputs('CDBAIRRO');
    const statusFieldMunicipality = this.getFilterInputs('CDMUNICIPIO');

    const neighborhoodFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldNeighborhood[0].name);

    if (component.value != null && component.name === statusFieldMunicipality[0].name) {
      neighborhoodFieldInstance.disabled = false;
      neighborhoodFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
        SGESTADO: statusFieldState[0].value,
        CDMUNICIPIO: statusFieldMunicipality[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-CDMUNICIPIO') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      const modalFieldNeighborhood = Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO');
      const modalFieldMunicipality = Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO');
      modalFieldNeighborhood.disabled = false;
      modalFieldNeighborhood.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
        SGESTADO: modalFieldState.value,
        CDMUNICIPIO: modalFieldMunicipality.value,
      });
    } else {
      neighborhoodFieldInstance.disabled = true;
      neighborhoodFieldInstance.value = null;
    }
  }

  /**
   * Displays a dialog box with buttons and a grid, to confirm whether the user
   * wants to find the address based on the information provided in the field.
   * @param {Object} component - The component used.
   * @param {Object} options - The options for the dialog.
   * @param {any} options.component - The component to be displayed in the dialog.
   */
  public showDialogWithButtonsGrid({ component }: any) {
    if (component.value.length === 8) {
      DialogService.show({
        name: 'dialog-find-partner-addres',
        type: 'warning',
        title: 'Buscar Endereço',
        text: 'Deseja buscar o endereço pelo CEP?',
        buttons: [
          {
            name: 'NAO',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'SIM',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                this.validateCEP(component.name);
                DialogService.hide();
              },
            },
          },
        ],
      });
    }
  }

  /**
   * Validates the CEP (postal code).
   * If the CEP is valid, it searches for the address details and updates the
   * corresponding form fields.
   */
  public async validateCEP(nameForm: string): Promise<void> {
    const CEP = Metadata.getInstance<TextInput>(`${nameForm}`).value.replace('-', '');
    if (CEP.length !== 8) {
      return;
    }

    LoadingService.show();
    try {
      const response = await this.searchCep(CEP) as IDictionary;
      const addressFound = response.data.dataset.ENDERECO;

      if (!addressFound) {
        return;
      }

      // Limpar campos de endereço antes de preencher com novos dados
      this.clearAddressFields();

      Metadata.getInstance<TextInput>('modal-cadastro-leads-DSENDERECO').value = addressFound.DSENDERECO;
      Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').setValue(addressFound.CDPAIS ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').setValue(addressFound.SGESTADO ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue(addressFound.CDBAIRRO ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').setValue(addressFound.CDMUNICIPIO ?? '');

      // Se o código do bairro estiver vazio, busca e preenche
      if (!addressFound.CDBAIRRO) {
        await this.fetchAndSetBairroCode(addressFound);
      }
    } catch (error) {
      Utilities.showErrorMessage(error);
    } finally {
      LoadingService.hide();
    }
  }

  /**
   * Clears the address fields in the modal form.
   */
  private clearAddressFields(): void {
    Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').setValue('');
  }

  /**
   * Fetches the bairro code based on the provided address information and sets
   * the value in the metadata.
   * @param addressFound - The address information containing NMBAIRRO, CDPAIS, SGESTADO, and CDMUNICIPIO.
   */
  private async fetchAndSetBairroCode(addressFound: IDictionary): Promise<void> {
    try {
      const bairroResponse = await Http.post('/findCodeDistrict', {
        NMBAIRRO: addressFound.NMBAIRRO,
        CDPAIS: addressFound.CDPAIS,
        SGESTADO: addressFound.SGESTADO,
        CDMUNICIPIO: addressFound.CDMUNICIPIO,
      });
      const bairroData = bairroResponse.data.data[0];
      const CDBAIRRO = bairroData ? bairroData.CDBAIRRO : '';

      Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue(CDBAIRRO);
    } catch (error) {
      Utilities.showErrorMessage(error);
    }
  }

  /**
   * Searches for address information based on a given CEP value.
   *
   * @param cepValue The CEP value to search for.
   * @returns A promise that resolves to the address information, or 0 if the CEP value is invalid.
   */
  private async searchCep(cepValue: number): Promise<any | number> {
    if (String(cepValue).length === 8) {
      try {
        const response = await Http.post('/lib_findAddress', {
          requestType: 'Row',
          row: { CEP: cepValue },
        });
        return response;
      } catch (error) {
        console.error('Error searching for address:', error);
        return 0;
      }
    }
    return 0;
  }

  /**
   * Saves a new lead.
   *
   * This method sends a POST request to the '/saveNewLeads' endpoint with the lead data.
   * If the form is validated successfully, it sends the lead data to the server to be saved.
   * Upon successful saving, it closes the modal window, refreshes the leads grid, and displays a success message.
   * If an error occurs during the process, it displays an error message.
   */
  public saveNewLead(): Promise<void> {
    const form = this.form('modal-cadastro-leads-form');
    const grid = this.obtainTekTreeGrid('tek_grid_crm_leads');

    if (Utilities.validateForm(form)) {
      LoadingService.show();
      Http.post('/saveNewLeads', {
        NMCANDIDATO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NMCANDIDATO').value,
        NMFANTCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NMFANTCAN').value,
        NRINSJURCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRINSJURCAN').value,
        IDTPIJURCAN: Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value,
        IDATIVO: Metadata.getInstance<Select>('modal-cadastro-leads-IDATIVO').value,
        IDSTATUSCAN: Metadata.getInstance<Select>('modal-cadastro-leads-IDSTATUSCAN').value,
        DSEMAILCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSEMAILCAN').value,
        DSTELEFONECAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSTELEFONECAN').value,
        PERFATUSEGMER: Metadata.getInstance<TextInput>('modal-cadastro-leads-PERFATUSEGMER').value,
        CDSEGMER: Metadata.getInstance<Select>('modal-cadastro-leads-CDSEGMER').value,
        NRFUNCCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRFUNCCAN').value,
        NRSISATU: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRSISATU').value,
        NRCEPENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRCEPENDERECO').value,
        DSENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSENDERECO').value,
        NRIMOVELENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRIMOVELENDERECO').value,
        DSCOMPLEENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSCOMPLEENDERECO').value,
        CDPAIS: Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').value,
        SGESTADO: Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').value,
        CDMUNICIPIO: Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').value,
        NMBAIRROCAN: Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').displayValue,
        CDBAIRRO: Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').value,
        NMINTERLOCUTOR: Metadata.getInstance<Select>('modal-cadastro-leads-NMINTERLOCUTOR').value,
        EMAILINTERLOC: Metadata.getInstance<TextInput>('modal-cadastro-leads-EMAILINTERLOC').value,
        NRTELEFONE1: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRTELEFONE1').value,
        CDOPERADOR: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRPARCPROPRI').value,
        NRPARCPROPRI: Metadata.getInstance<Select>('modal-cadastro-leads-NRPARCPROPRI').datasource.data[0].NRPARCRESP,
        NRSEQPERFATIV: Metadata.getInstance<Select>('modal-cadastro-leads-text-NRSEQPERFATIV').value,
        CDMODODIVULG: Metadata.getInstance<Select>('modal-cadastro-leads-text-CDMODODIVULG').value,
        NRMODORECEBINFO: Metadata.getInstance<Select>('modal-cadastro-leads-text-NRMODORECEBINFO').value,
        DESCTITULOCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DESCTITULOCAN').value,
        DSOBSERVACAOLEAD: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSOBSERVACAOLEAD').value,
        PORTCANDIDATO: Metadata.getInstance<TextInput>('modal-cadastro-leads-PORTCANDIDATO').value,
        QNTDUSUARIOS: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUSUARIOS').value,
        QNTDUNIDADES: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').value,
        QNTPONTOSACESSO: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').value,
        FISCAL: Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').value,
        CDMODATEND: Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').value,
        NRREFEICOESDIA: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').value,
      }).then(() => {
        this.closeModal();
        grid.datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }
    return Promise.resolve();
  }

  private getOperatorWithMostRecentOpenTask(options: any): string {
    options.sort((a: any, b: any) => {
      const dateA = moment(a.DTPLANFILACONTA);
      const dateB = moment(b.DTPLANFILACONTA);
      return dateB.valueOf() - dateA.valueOf();
    });
    const latestOption = options[0];
    const { NMOPERADOR } = latestOption;
    return NMOPERADOR;
  }

  private showDialogEvolutionModal(component: any, options: any): void {
    const NMOPERADOR = this.getOperatorWithMostRecentOpenTask(options);
    if (component.name === 'modal-register-evolution-form') {
      DialogService.show({
        name: 'dialog-find-partner-addres',
        type: 'warning',
        title: 'Tarefas Em Aberto',
        text: `Este parceiro possui tarefas "Em Aberto" para o seguinte operador:<br/> <br> ${NMOPERADOR} <br/> <br>Deseja continuar?<br/>`,
        buttons: [
          {
            name: 'NAO',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'SIM',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                this.hasNextTask();
                DialogService.hide();
              },
            },
          },
        ],
      });
    }
  }

  private hasNextTask(): void {
    const dateFromNextTask = Metadata.getInstance<Select>('modal-register-evolution-DTPLANFILACONTA').value;
    if (dateFromNextTask) {
      this.saveNewTaskForLead();
    } else {
      this.saveNewEvolution();
    }
  }

  public hasOpenTaskForSaveNewEvolution(): Promise<void> {
    const form = this.form('modal-register-evolution-form');

    if (Utilities.validateForm(form)) {
      LoadingService.show();
      Http.post('/checkIfThereAreAnyOpenTasks', {
        NRPARCNEGOCIO: this.currentRow.NRPARCNEGOCIO,
      }).then((response) => {
        if (response.data.data.length > 0) {
          this.showDialogEvolutionModal(form, response.data.data);
        } else {
          this.hasNextTask();
        }
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }
    return Promise.resolve();
  }

  private saveNewEvolution(nrSeqFilaContato: any = null): Promise<void> {
    const grid = this.obtainTekTreeGrid('tekgrid-tab-evolution');
    const form = this.form('modal-register-evolution-form');

    if (Utilities.validateForm(form)) {
      LoadingService.show();

      const requestBody = {
        NRSEQFILACONTATO: nrSeqFilaContato?.data ?? null,
        NRCONTATO: this.getFieldBySelectedLead(grid, 'NRCONTATO'),
        NRSEQCONTATO: this.getFieldBySelectedLead(grid, 'NRSEQCONTATO'),
        IDTPEVOLCONT: this.getFieldBySelectedLead(grid, 'IDTPEVOLCONT') ?? 'P',
        IDGERAFATURA: this.getFieldBySelectedLead(grid, 'IDGERAFATURA') ?? 'NAO',
        IDANEXOEVOLCONT: this.getFieldBySelectedLead(grid, 'IDANEXOEVOLCONT') ?? 'N',
        NRRELACIONAPARC: this.getFieldBySelectedLead(grid, 'NRRELACIONAPARC') ?? null,
        CDOPEREVOL: Metadata.getInstance<Select>('modal-register-evolution-CDOPEREVOL').value,
        TXEVOLCONT: Metadata.getInstance<Select>('modal-register-evolution-TXEVOLCONT').value,
        CDOPERREAL: Metadata.getInstance<Select>('modal-register-evolution-CDOPERREAL').value,
        CDTIPOCONT: Metadata.getInstance<Select>('modal-register-evolution-CDTIPOCONT').value,
        CDMOTIEVOCON: Metadata.getInstance<Select>('modal-register-evolution-CDMOTIEVOCON').value,
        DTHREVOLCONT: Metadata.getInstance<TextInput>('modal-register-evolution-DTHREVOLCONT').value,
        IDSENEVOCONT: Metadata.getInstance<TextInput>('modal-register-evolution-IDSENEVOCONT').value,
        NRSEQINTERLOCUTOR: Metadata.getInstance<Select>('modal-register-evolution-NMINTEVOCONT').value,
        NMINTEVOCONT: Metadata.getInstance<Select>('modal-register-evolution-NMINTEVOCONT').displayValue,
        DSFILACONTATO: Metadata.getInstance<TextInput>('modal-register-evolution-DSFILACONTATO').value,
        NRSEQRESULEVOL: Metadata.getInstance<TextInput>('modal-register-evolution-NRSEQRESULEVOL').value,
        NRPRIORFILACONT: Metadata.getInstance<TextInput>('modal-register-evolution-NRPRIORFILACONT').value,
        NRSEQCLASSIFICATAREFA: Metadata.getInstance<Select>('modal-register-evolution-NRSEQCLASSIFICATAREFA').value,
      };

      return Http.post('/saveNewEvolution', requestBody)
        .then(() => {
          this.closeModal();
          grid.datasource.get();
          Utilities.showSuccessMessage();
        })
        .catch((error: any) => {
          Utilities.showErrorMessage(error);
        })
        .finally(() => {
          LoadingService.hide();
        });
    }

    return Promise.resolve();
  }

  private getFieldBySelectedLead(grid: any, fieldName: string): any {
    return grid.datasource.data[0]?.[fieldName] ?? this.currentRow[fieldName] ?? null;
  }

  private saveNewTaskForLead(): Promise<void> {
    const grid = this.obtainTekTreeGrid('tekgrid-tab-evolution');
    const form = this.form('modal-register-evolution-form');

    if (Utilities.validateForm(form)) {
      LoadingService.show();

      const requestBody = {
        NRCONTATO: this.getFieldBySelectedLead(grid, 'NRCONTATO'),
        NRSEQCONTATO: this.getFieldBySelectedLead(grid, 'NRSEQCONTATO'),
        NRPARCNEGOCIO: this.getFieldBySelectedLead(grid, 'NRPARCNEGOCIO'),
        DSEMAILTAREFA: this.getFieldBySelectedLead(grid, 'EMAILINTERLOC'),
        DSTELEFONETAREFA: this.getFieldBySelectedLead(grid, 'NRTELEFONE1'),
        NRSEQINTERLOCUTOR: this.getFieldBySelectedLead(grid, 'NRSEQINTERLOCUTOR'),
        CDOPERREAL: Metadata.getInstance<Select>('modal-register-evolution-CDOPERREAL').value,
        DSFILACONTATO: Metadata.getInstance<Select>('modal-register-evolution-DSFILACONTATO').value,
        DTPLANFILACONTA: Metadata.getInstance<Select>('modal-register-evolution-DTPLANFILACONTA').value,
        NRPRIORFILACONT: Metadata.getInstance<TextInput>('modal-register-evolution-NRPRIORFILACONT').value,
        NRSEQCLASSIFICATAREFA: Metadata.getInstance<Select>('modal-register-evolution-NRSEQCLASSIFICATAREFA').value,
      };

      return Http.post('/insertLeadEvolNextTask', requestBody)
        .then((response) => {
          this.saveNewEvolution(response);
          Utilities.showSuccessMessage();
        })
        .catch((error: any) => {
          Utilities.showErrorMessage(error);
        })
        .finally(() => {
          LoadingService.hide();
        });
    }

    return Promise.resolve();
  }

  /**
  * Changes visibility of fields based on the selected profile.
  *
  * @param component The component containing the selected profile.
  * @returns void
  */
  public changingVisibilityBasedOnProfile({ component }: any): void {
    const activityProfile = component.value;

    if (activityProfile === '2') {
      this.showProfileRetailFields();
    } else if (activityProfile === '4') {
      this.showTecFoodFields();
    } else {
      this.hideAllFields();
    }
  }

  /**
  * Shows fields specific to the retail profile.
  *
  * @returns void
  */
  private showProfileRetailFields(): void {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').isVisible = true;
  }

  /**
  * Shows fields specific to the Tecfood profile.
  *
  * @returns void
  */
  private showTecFoodFields(): void {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = true;
  }

  /**
  * Hides all fields when this option is not a 2 or 4.
  *
  * @returns void
  */
  private hideAllFields(): void {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').isVisible = false;
  }

  /**
   * Modifies the visibility of buttons based on the status of a lead.
   * @param row The lead object containing status information.
   */
  public modifyVisibilityBasedOnLeadStatus(row: any): void {
    const statusLead = row;

    // Set the visibility of the buttons based on the lead status
    if (statusLead.IDSTATUSCAN === '3') {
      this.setButtonVisibility(false, true);
    } else if (statusLead.IDSTATUSCAN === '4') {
      this.setButtonVisibility(true, false);
    } else {
      this.setButtonVisibility(true, true);
    }
  }

  // Helper method to set the visibility of the buttons
  private setButtonVisibility(qualifiedVisibility: boolean, desqualifiedVisibility: boolean): void {
    this.buttonQualifiedyVisibility = qualifiedVisibility;
    this.buttonDesqualifiedyVisibility = desqualifiedVisibility;
  }

  /**
   * Changes the status of a lead based on the provided component.
   * If the component represents the action to qualify a lead,
   * the lead status will be updated to 'Qualified' (IDSTATUSCAN: '3').
   * If the component represents the action to disqualify a lead,
   * the lead status will be updated to 'Disqualified' (IDSTATUSCAN: '4').
   * @param {any} { component } The component triggering the status change action.
   */
  public changeLeadStatus({ component }: any) {
    const isQualifyAction = (component.name === 'actionQualifyLead');
    const newLeadStatus = isQualifyAction ? '3' : '4';

    this.updateLeadStatus(newLeadStatus);
  }

  /**
   * Updates the status of a lead with the specified new status.
   * @param newStatus The new status to set for the lead.
   * @returns A promise that resolves when the lead status is updated.
   */
  private updateLeadStatus(newStatus: string): Promise<void> {
    // Show loading indicator
    LoadingService.show();

    // Construct request payload
    const requestData = {
      NRPARCNEGOCIO: this.currentRow.NRPARCNEGOCIO,
      IDSTATUSCAN: newStatus,
      NRORG: this.currentRow.NRORG,
      CRM_NRORG: this.currentRow.NRORG,
      CDOPERULTATU: this.currentRow.CDOPERADOR,
    };

    // Send POST request to update lead status
    return Http.post('/changeStatusLead', requestData)
      .then(() => {
        // Show success message
        Utilities.showSuccessMessage();

        // Refresh grid data source
        this.getGridInstance().datasource.get();

        // Set new status value in the select input
        Metadata.getInstance<Select>('formCandidateData-IDSTATUSCAN').setValue(newStatus);
      })
      .catch((error: any) => {
        // Show error message
        Utilities.showErrorMessage(error);
        throw error; // Propagate the error to the caller
      })
      .finally(() => {
        // Hide loading indicator
        LoadingService.hide();
      });
  }

  public getDateTimeNow({ component }: any): void {
    Metadata.getInstance<Date>(component.name).value = moment().format('DD/MM/YYYY HH:mm');
  }

  /**
   * Sets the display style of the specified component to 'flex'.
   * @param componentName The ID of the component whose display style will be set to 'flex'.
   */
  public setDisplayFlex(componentName: string) {
    // Wait for the next event loop iteration before updating the style
    setTimeout(() => {
      const component = document.getElementById(componentName);
      if (component) {
        component.style.display = 'flex';
      } else {
        console.error(`Component "${componentName}" not found.`);
      }
    }, 0);
  }
}
